<?php
namespace Pinpoint\CheckoutSuccess\Block\Onepage;

use Magento\Catalog\Block\Product\Image;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;

class Success extends \Magento\Framework\View\Element\Template
{
    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;
    /**
     * @var \Magento\Catalog\Block\Product\ImageBuilder
     */
    protected $imageBuilder;
    /**
     * @var \Magento\Framework\Pricing\PriceCurrencyInterface
     */
    protected $priceCurrency;
    /**
     * @var \Magento\Sales\Model\Order\Address\Renderer
     */
    protected $renderer;
    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    private $productRepository;

    public function __construct(
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Catalog\Block\Product\ImageBuilder $imageBuilder,
        \Magento\Framework\Pricing\PriceCurrencyInterface $priceCurrency,
        \Magento\Sales\Model\Order\Address\Renderer $renderer,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->_checkoutSession = $checkoutSession;
        $this->orderRepository = $orderRepository;
        $this->imageBuilder = $imageBuilder;
        $this->priceCurrency = $priceCurrency;
        $this->renderer = $renderer;
        $this->productRepository = $productRepository;
    }

    /**
     * @return string|IncrementId
     */
    public function getRealOrderId()
    {
        $lastOrderId = $this->_checkoutSession->getLastOrderId();
        $order = $this->orderRepository->get($lastOrderId);
        return $order->getIncrementId();
    }

    /**
     * @return \Magento\Sales\Model\Order
     */
    public function getLastOrder()
    {
        return $this->_checkoutSession->getLastRealOrder();
    }

    /**
     * Retrieve product image
     * @param $item
     * @param $imageId
     * @param array $attributes
     * @return Image
     * @throws NoSuchEntityException
     */
    public function getImage($item, $imageId, $attributes = [])
    {
        $product = $this->getProductFromItem($item);
        if ($product->getSmallImage() && $product->getSmallImage() !== 'no_selection') {
            return $this->imageBuilder
                ->setProduct($product)
                ->setImageId($imageId)
                ->setAttributes($attributes)
                ->create();
        }

        return $this->imageBuilder
            ->setProduct($item->getProduct())
            ->setImageId($imageId)
            ->setAttributes($attributes)
            ->create();
    }

    /**
     * Return selected simple product if $_item has options
     *
     * @param $item
     * @return \Magento\Catalog\Api\Data\ProductInterface
     * @throws NoSuchEntityException
     */
    private function getProductFromItem($item)
    {
        $options = $item->getProductOptions();
        if (isset($options['simple_sku'])) {
            return $this->productRepository->get($options['simple_sku']);
        }

        return $item->getProduct();
    }

    /**
     * Return item unit price html
     * @param null $item
     * @return string
     * @throws LocalizedException
     */
    public function getItemPriceHtml($item = null)
    {
        $block = $this->getLayout()->getBlock('item_unit_price');
        if (!$item) {
            $item = $this->getItem();
        }
        $block->setItem($item);
        return $block->toHtml();
    }

    /**
     * @param $address
     * @return null|string
     */
    public function getFormattedAddress($address)
    {
        return $this->renderer->format($address, 'html');
    }

    /**
     * @param $address
     * @param $fields
     * @return string
     */
    public function addressToString($address, $fields)
    {
        $string = '';
        foreach ($fields as $code) {
            $string .= $address->getData($code) . ', ';
        }

        return trim($string, ', ');
    }

    /**
     * @param $price
     * @return float
     */
    public function getCurrencyFormat($price)
    {
        return $this->priceCurrency->format($price, true, 2);
    }

    /**
     * @param $order
     * @return mixed
     */
    public function getPaymentMethodTitle($order)
    {
        $payment = $order->getPayment();
        $method = $payment->getMethodInstance();
        return $method->getTitle();
    }

    /**
     * @return mixed
     */
    public function getContinueUrl()
    {
        try {
            return $this->_storeManager->getStore()->getBaseUrl();
        } catch (NoSuchEntityException $e) {
        }
    }
}
